import { PureComponent, FC, HTMLAttributes, InputHTMLAttributes, ChangeEvent, MouseEvent, DragEvent, ReactNode, KeyboardEvent, FormEvent, Component, PropsWithChildren } from "react";
import { ImageProps, FloatingLabelProps, FormControlProps, TableProps } from "react-bootstrap";
import { FormComponent, FormComponentProps } from "mobx-react-helper";
import { BaseModel, DataObject, IDType, ListModel, Filter } from "mobx-restful";
import { TranslationModel } from "mobx-i18n";
export class ImagePreview extends PureComponent<ImageProps> {
    accessor downloading: boolean;
    accessor loadedPath: string;
    accessor viewing: boolean;
    componentDidMount(): void;
    componentDidUpdate({ src }: Readonly<ImageProps>): void;
    load(path: string): Promise<void>;
    render(): import("react/jsx-runtime").JSX.Element;
}
export type FilePreviewProps = ImageProps & HTMLAttributes<HTMLAudioElement> & HTMLAttributes<HTMLVideoElement> & HTMLAttributes<HTMLAnchorElement> & {
    type?: InputHTMLAttributes<HTMLInputElement>['accept'];
    path: string;
};
export const FileTypeMap: {
    stream: string;
    compressed: string;
    msword: string;
    document: string;
    powerpoint: string;
    presentation: string;
    excel: string;
    sheet: string;
};
export const FilePreview: FC<FilePreviewProps>;
export interface FilePickerProps extends FormComponentProps {
    onChange?: (value: FormComponentProps['value'], file?: File) => any;
}
export class FilePicker extends FormComponent<FilePickerProps> {
    static readonly displayName = "FilePicker";
    handleAdd: (event: ChangeEvent<HTMLInputElement>) => void;
    handleClear: (event: MouseEvent<HTMLButtonElement>) => void;
    renderInput(): import("react/jsx-runtime").JSX.Element;
    render(): import("react/jsx-runtime").JSX.Element;
}
export abstract class FileModel extends BaseModel {
    accessor files: string[];
    clear(): void;
    /**
     * Override this method for Network calling,
     * then call `super.upload(fileURL)` to update `this.files` array.
     */
    upload(file: string | Blob): Promise<string>;
    /**
     * Override this method for Network calling,
     * then call `super.delete(fileURL)` to update `this.files` array.
     */
    delete(file: string): Promise<void>;
    move(sourceIndex: number, targetIndex: number): void;
}
export interface FileUploaderProps extends FormComponentProps {
    store: FileModel;
}
export class FileUploader extends FormComponent<FileUploaderProps> {
    accessor pickIndex: number | undefined;
    componentDidMount(): void;
    componentDidUpdate(prevProps: Readonly<FileUploaderProps>): void;
    handleDrop: (index: number) => (event: DragEvent<HTMLElement>) => void;
    handleChange: (oldURI?: string) => (URI: string, file?: File) => Promise<void>;
    render(): import("react/jsx-runtime").JSX.Element;
}
export type FormFieldProps = InputHTMLAttributes<HTMLInputElement> & FormControlProps & Pick<FloatingLabelProps, 'label'>;
export const FormField: FC<FormFieldProps>;
export interface RangeInputProps extends Omit<FormComponentProps, 'type'> {
    icon?: ReactNode | ((value: number) => ReactNode);
}
export class RangeInput extends FormComponent<RangeInputProps> {
    observedProps: RangeInputProps;
    handleChange: ({ currentTarget: { value }, }: ChangeEvent<HTMLInputElement>) => void;
    render(): import("react/jsx-runtime").JSX.Element;
}
export const TextInputTypes: readonly ["text", "number", "tel", "email", "url"];
export interface BadgeInputProps extends Pick<InputHTMLAttributes<HTMLInputElement>, 'className' | 'style' | 'name' | 'required' | 'placeholder'> {
    type?: (typeof TextInputTypes)[number];
    defaultValue?: string[];
    value?: string[];
    onChange?: (value: string[]) => any;
}
export class BadgeInput extends PureComponent<BadgeInputProps> {
    static match(type: string): type is BadgeInputProps['type'];
    accessor innerValue: string[];
    observedProps: BadgeInputProps;
    get value(): string[];
    handleInput: (event: KeyboardEvent<HTMLInputElement>) => void;
    delete(index: number): void;
    render(): import("react/jsx-runtime").JSX.Element;
}
export interface Field<T extends DataObject> extends Pick<InputHTMLAttributes<HTMLInputElement>, 'type' | 'readOnly' | 'required' | 'min' | 'minLength' | 'max' | 'maxLength' | 'step' | 'multiple' | 'accept' | 'placeholder'> {
    key?: keyof T;
    renderLabel?: ReactNode | ((data: keyof T) => ReactNode);
    renderInput?: (data: T, meta: Field<T>) => ReactNode;
}
export interface RestFormProps<T extends DataObject> {
    id?: IDType;
    fields: Field<T>[];
    store: ListModel<T>;
    translator: TranslationModel<string, 'submit' | 'cancel'>;
    uploader?: FileModel;
}
export class RestForm<T extends DataObject> extends PureComponent<RestFormProps<T>> {
    componentDidMount(): void;
    handleSubmit: (event: FormEvent<HTMLFormElement>) => Promise<void>;
    get fields(): Field<T>[];
    renderInput: ({ key, renderLabel, renderInput, ...props }: Field<T>) => string | number | true | Iterable<ReactNode> | import("react/jsx-runtime").JSX.Element;
    render(): import("react/jsx-runtime").JSX.Element;
}
export type PageMeta = Pick<ListModel<{}>, 'pageSize' | 'pageIndex'>;
export interface PagerProps extends PageMeta {
    pageCount: number;
    onChange?: (meta: PageMeta) => any;
}
export const Pager: FC<PagerProps>;
export interface Column<T extends DataObject> extends Omit<Field<T>, 'renderLabel'> {
    renderHead?: Field<T>['renderLabel'];
    renderBody?: (data: T) => ReactNode;
    renderFoot?: ReactNode | ((data: keyof T) => ReactNode);
}
export interface RestTableProps<T extends DataObject> extends TableProps, Omit<RestFormProps<T>, 'id' | 'fields' | 'translator'> {
    editable?: boolean;
    deletable?: boolean;
    columns: Column<T>[];
    translator: RestFormProps<T>['translator'] & TranslationModel<string, 'create' | 'edit' | 'delete' | 'total_x_rows' | 'sure_to_delete_x'>;
    onCheck?: (keys: IDType[]) => any;
}
export class RestTable<T extends DataObject> extends Component<RestTableProps<T>> {
    static displayName: string;
    componentDidMount(): void;
    observedProps: RestTableProps<T>;
    accessor checkedKeys: IDType[];
    toggleCheck(key: IDType): void;
    toggleCheckAll: () => void;
    get checkColumn(): Column<T>;
    get operateColumn(): Column<T>;
    get columns(): Column<T>[];
    get hasHeader(): boolean;
    get hasFooter(): boolean;
    get editing(): boolean;
    renderTable(): import("react/jsx-runtime").JSX.Element;
    renderDialog(): import("react/jsx-runtime").JSX.Element;
    getList: import("lodash").DebouncedFunc<({ pageIndex, pageSize }: any) => void>;
    deleteList(keys: IDType[]): Promise<void>;
    render(): import("react/jsx-runtime").JSX.Element;
}
export type EdgePosition = 'top' | 'bottom' | 'left' | 'right';
export type TouchHandler = (edge: EdgePosition) => any;
export type ScrollBoundaryProps = PropsWithChildren<Partial<Record<EdgePosition, ReactNode>> & {
    className?: string;
    onTouch: TouchHandler;
}>;
export const ScrollBoundary: FC<ScrollBoundaryProps>;
export interface ScrollListProps<D extends DataObject, F extends Filter<D> = Filter<D>> extends Pick<ScrollBoundaryProps, 'className'> {
    translator: TranslationModel<string, 'load_more' | 'no_more'>;
    store: ListModel<D, F>;
    filter?: F;
    defaultData?: D[];
    renderList(allItems: D[]): ReactNode;
}
export class ScrollList<D extends DataObject = DataObject, F extends Filter<D> = Filter<D>> extends Component<ScrollListProps<D, F>> {
    componentDidMount(): Promise<void>;
    componentWillUnmount(): void;
    loadMore: import("lodash").DebouncedFunc<(edge: EdgePosition) => void>;
    render(): import("react/jsx-runtime").JSX.Element;
}

//# sourceMappingURL=index.d.ts.map
